IMPLEMENTATION MODULE Event ;

FROM Windows IMPORT
     HANDLE, NULL,
     DefWindowProc;

FROM Storage IMPORT 
     ALLOCATE, DEALLOCATE, Available ;

TYPE
     ELEMENT  = RECORD 
                event    : EVENT ;
                response : RESPONSE ;
                END ; 
                        
     PtrToELEMENT = POINTER TO ARRAY [1..MAXEVENT] OF ELEMENT ;                     
     
     LIST     = POINTER TO RECORD
                end  : CARDINAL ; (* Index to last element. *)
                size : CARDINAL ; (* Number of elements allocated. *)
                data : PtrToELEMENT ; 
                END ;                             

(*--------------------------------------------------------*)

PROCEDURE Init( Size : CARDINAL ) : LIST;

   VAR List : LIST;

   BEGIN
       IF Available( SIZE(LIST) + SIZE(ELEMENT)*Size ) THEN
          ALLOCATE( List, SIZE(LIST) ) ;
          List^.end  := 0 ;
          List^.size := Size ;
          ALLOCATE(List^.data, SIZE(ELEMENT) * Size) ;
          RETURN List ;
       ELSE
          RETURN NIL ;
       END; (* IF *)
   END Init;

(*--------------------------------------------------------*)

PROCEDURE Done( VAR List : LIST ) ;   
   BEGIN
       DEALLOCATE(List^.data, SIZE(ELEMENT) * List^.size ) ;
       DEALLOCATE(List, SIZE(LIST) ) ;
   END Done;

(*--------------------------------------------------------*)
   
PROCEDURE Register( VAR List : LIST;
                    event    : EVENT;
                    response : RESPONSE ) : BOOLEAN;
   BEGIN
       WITH List^ DO
            IF end < size THEN
               INC(end) ;
               data^[end].event := event;
               data^[end].response := response;
               RETURN TRUE ;
            ELSE
               RETURN FALSE ;
            END; (* if *)
       END; (* with *)
   END Register;

(*--------------------------------------------------------*)
   
PROCEDURE Respond( List         : LIST;
                   hWnd         : HANDLE; 
                   wMsg, wParam : CARDINAL;
                   lParam       : LONGINT ) : LONGINT ;
   VAR 
      i : CARDINAL ;

   BEGIN
        WITH List^ DO
             FOR i:= 1 TO end DO
                 IF data^[i].event = wMsg THEN
                    data^[i].response(hWnd,wMsg,wParam,lParam);
                    RETURN NULL ;
                 END; (* if *)
             END; (* for *)
             RETURN DefWindowProc( hWnd, wMsg, wParam, lParam);
        END; (* with *)
   END Respond;   
   
END Event.
